# Author: Quinten Van Der Post
import plotly.graph_objs as go
from plotly.offline import plot
import sys

import Protein

def plotly_vis(protein):
    tuple_array = [eval(coords) for coords in protein.make_grid().keys()]
    type_array = protein.elements

    x_values = []
    y_values = []
    z_values = []
    for coords in tuple_array:
        x_values.append(coords[0])
        y_values.append(coords[1])
        if protein.dimension == 3:
            z_values.append(coords[2])
        else:
            z_values.append(0)

    color_dict = {
        "P": "blue",
        "H": "red",
        "C": "yellow"
    }
    color_array = [color_dict[x] for x in type_array]

    trace = go.Scatter3d(
        x=x_values, y=y_values, z=z_values, text=type_array, name="Protein",
        mode="lines+markers+text",
        marker=dict(
            size=8,
            color=color_array,
        ),
        line=dict(
            color='black',
            width=1,
        ),
        textfont=dict(
            family='sans serif',
            size=18,
            color='black'
        ),
        showlegend=False,
    )

    data = [trace]

    min_x = min(x_values)
    max_x = max(x_values)
    min_y = min(y_values)
    max_y = max(y_values)
    min_z = min(z_values)
    max_z = max(z_values)
    min_range = min((min_x, min_y)) - 1
    max_range = max((max_x, max_y)) + 1
    min_range = min((min_x, min_y, min_z)) - 1
    max_range = max((max_x, max_y, max_z)) + 1

    layout = dict(
        autosize=True,
        title='Algorithmic Protein Folding',
        scene=dict(
            xaxis=dict(
                range=[min_range, max_range],
                gridcolor='rgb(255, 255, 255)',
                zerolinecolor='rgb(255, 255, 255)',
                showbackground=True,
                zeroline=False,
                ticks="",
                showticklabels=False,
                backgroundcolor='rgb(230, 230,230)'
            ),
            yaxis=dict(
                range=[min_range, max_range],
                gridcolor='rgb(255, 255, 255)',
                zerolinecolor='rgb(255, 255, 255)',
                showbackground=True,
                zeroline=False,
                ticks="",
                showticklabels=False,
                backgroundcolor='rgb(230, 230,230)'
            ),
            zaxis=dict(
                range=[min_range, max_range],
                gridcolor='rgb(255, 255, 255)',
                zerolinecolor='rgb(255, 255, 255)',
                showbackground=True,
                zeroline=False,
                ticks="",
                showticklabels=False,
                backgroundcolor='rgb(230, 230,230)'
            ),
            camera=dict(
                up=dict(
                    x=0,
                    y=0,
                    z=1
                ),
                eye=dict(
                    x=-1.7428,
                    y=1.0707,
                    z=0.7100,
                )
            ),
            aspectratio=dict(x=1, y=1, z=1),
            aspectmode='manual'
        ),
    )

    fig = dict(data=data, layout=layout)

    plot(fig, filename="output.html")


if __name__== "__main__":
    if len(sys.argv)<2:
        print("usage: python "+str(sys.argv[0])+" <filename>")
        print("Visualizes the folded protein in the given file.")
        sys.exit()
    filename=sys.argv[1]
    print("reading file '"+filename+"':")
    protein=Protein.Protein(filename)
    print(protein)
    plotly_vis(protein)
